--[[-------------------------------------------------------------------------
  Copyright (c) 2010, Kyahx
  All rights reserved.
---------------------------------------------------------------------------]]

local defaults = {
	useBonus = true,
	header = '[mod:ctrl-alt] 2; [mod:alt-shift] 4; [mod:shift] 5; [mod:alt] 6; [mod:ctrl] 3;',
}
PagemasterDB = PagemasterDB or defaults

local PM = CreateFrame('Frame', nil, MainMenuBarArtFrame, 'SecureHandlerStateTemplate')
PM:SetFrameLevel(3)
PM:SetAttribute('_onstate-page', [[ control:ChildUpdate('action', newstate) ]])

PM.db = PagemasterDB

local stateDriver = function()
	return '[bonusbar:5]11; '..PM.db.header..(PM.db.useBonus and ' [bonusbar:1] 7; [bonusbar:2] 8; [bonusbar:3] 9; [bonusbar:4] 10;' or '')..' 1'
end

RegisterStateDriver(PM, 'page', stateDriver())

-- Hijack the default 12 ActionButtons
for id=1,12 do
	local ab = _G['ActionButton'..id]
	
	ab:SetParent(PM)
	
	ab:SetAttribute('baseID', ab:GetID())
	ab:SetAttribute('action', ab:GetID())
	ab:SetID(0)
	
	ab:SetAttribute('useparent-actionpage', nil)
	ab:SetAttribute('useparent-unit', true)
	
	ab:SetAttribute('_childupdate-action', [[
		local id = self:GetAttribute('baseID') + ((message - 1) * 12)
		self:SetAttribute('action', id)
	]])
end

-- Hide the BonusActionBar (form switches are handled by the new state driver)
BonusActionBarFrame:Hide()
BonusActionBarFrame.Show = function() end

--[[-------------------------------------------------------------------------
  GUI Center
---------------------------------------------------------------------------]]
local EDGEGAP, GAP = 16, 8

local frame = CreateFrame('Frame', nil, InterfaceOptionsFramePanelContainer)
frame.name = 'Pagemaster'
frame:Hide()
frame:SetScript('OnShow', function(frame)
	local title, subtitle = LibStub('tekKonfig-Heading').new(frame, 'Pagemaster', 'This panel allows you to configure Pagemaster\'s State Header.')
	
	local stateLabel = frame:CreateFontString(nil, 'OVERLAY', 'GameFontNormalSmall')
	stateLabel:SetText('State Driver String')
	stateLabel:SetPoint('TOPLEFT', subtitle, 'BOTTOMLEFT', 5, -GAP)
	
	local YOFFSET = (stateLabel:GetTop() - frame:GetBottom() - EDGEGAP/3)/2
	local backdrop = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	}

	local stateEditbox = CreateFrame('EditBox', nil, frame)

	local SaveState = function()
		PM.db.header = gsub(stateEditbox:GetText(), '\n', ' ')
		RegisterStateDriver(PM, 'page', stateDriver())
	end

	stateEditbox:SetPoint('TOP', stateLabel, 'BOTTOM', 0, -5)
	stateEditbox:SetPoint('LEFT', EDGEGAP, 0)
	stateEditbox:SetPoint('BOTTOMRIGHT', -EDGEGAP, YOFFSET + EDGEGAP)
	stateEditbox:SetFontObject(GameFontHighlight)
	stateEditbox:SetTextInsets(8,8,8,8)
	stateEditbox:SetBackdrop(backdrop)
	stateEditbox:SetBackdropColor(0.1, 0.1, 0.1)
	stateEditbox:SetBackdropBorderColor(0.4, 0.4, 0.4)
	stateEditbox:SetMultiLine(true)
	stateEditbox:SetAutoFocus(false)
	stateEditbox:SetText(gsub(PM.db.header, '; ', ';\n'))
	stateEditbox:SetScript('OnEditFocusLost', SaveState)
	stateEditbox:SetScript('OnEscapePressed', stateEditbox.ClearFocus)
	
	local saveButton = LibStub('tekKonfig-Button').new(stateEditbox, 'BOTTOMRIGHT', stateEditbox, 'BOTTOMRIGHT', -GAP, GAP)
	saveButton:SetText('Save')
	saveButton:SetScript('OnClick', SaveState)
	saveButton:SetScale(0.8)
	if InCombatLockdown() then saveButton:Disable() end
	
	local resetButton = LibStub('tekKonfig-Button').new(stateEditbox, 'RIGHT', saveButton, 'LEFT', -GAP/2, 0)
	resetButton:SetText('Reset')
	resetButton:SetScript('OnClick', function()
		stateEditbox:SetText(gsub(defaults.header, '; ', ';\n'))
		SaveState()
	end)
	resetButton:SetScale(0.8)
	if InCombatLockdown() then resetButton:Disable() end
	
	frame:SetScript('OnShow', nil)
end)

InterfaceOptions_AddCategory(frame)

LibStub('tekKonfig-AboutPanel').new('Pagemaster', 'Pagemaster')